import stan
import arviz as az
import scipy.stats as ss
import matplotlib.pyplot as plt

Data = ss.multivariate_normal.rvs(mean = [0,0], cov = [[1,0.7],[0.7,1]],
                                  size = 100)

model_script = """
    data {
        int N;
        array[N] vector[2] XY;
    }
    parameters {
        real mu1;
        real mu2;
        real<lower = 0> v_sgm1;
        real<lower = 0> v_sgm2;
        real<lower=-1, upper=1> rho;
    }
    transformed parameters {
        vector[2] mus;
        matrix[2,2] cov;
        real sgm1;
        real sgm2;
        sgm1 = v_sgm1 * 10.0;
        sgm2 = v_sgm2 * 10.0;
        mus[1] = mu1;
        mus[2] = mu2;
        cov[1][1] = sgm1^2;
        cov[2][2] = sgm2^2;
        cov[1][2] = sgm1*sgm2*rho;
        cov[2][1] = cov[1][2];
    }
    model {
        mu1 ~ normal(0,10);
        mu2 ~ normal(0,10);
        v_sgm1 ~ exponential(0.1);
        v_sgm2 ~ exponential(0.1);
        rho ~ uniform(-1, 1);
        XY ~ multi_normal(mus, cov);
    }
"""

Stan_Data = {'N':len(Data), 'XY':Data}
sm = stan.build(model_script, data = Stan_Data)
fit = sm.sample()
#
#      Convert sampling data into an InferenceData class
#
i_data = az.from_pystan(posterior=fit, posterior_model=sm)
smry = az.summary(i_data)
print('Summary...\n', smry)

az.plot_trace(i_data, var_names = ['mu1', 'mu2', 'sgm1', 'sgm2', 'rho'])
plt.tight_layout()
plt.savefig('FigTrace.png')
plt.show()

az.plot_posterior(i_data, point_estimate = 'median', var_names = ['rho'])
plt.savefig('FigRho.png')
plt.show()

